/*
 * Copyright 2020 Rockchip Electronics S.LSI Co. LTD
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

#ifndef __TOYBRICK_LOG_H__
#define __TOYBRICK_LOG_H__

#if __ANDROID__
#ifndef LOG_TAG
#define LOG_TAG "toybrick"
#endif

#include <android/log.h>
#define ALOGD(...) __android_log_print(ANDROID_LOG_DEBUG, LOG_TAG, __VA_ARGS__)
#define ALOGI(...) __android_log_print(ANDROID_LOG_INFO, LOG_TAG, __VA_ARGS__)
#define ALOGW(...) __android_log_print(ANDROID_LOG_WARN, LOG_TAG, __VA_ARGS__)
#define ALOGE(...) __android_log_print(ANDROID_LOG_ERROR, LOG_TAG, __VA_ARGS__)

#define toybrick_debug  ALOGD
#define toybrick_info	ALOGI
#define toybrick_warn	ALOGW
#define toybrick_err	ALOGE

#else

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>

#define TOYBRICK_DEFAULT_MASK        0x0c

enum {
	TOYBRICK_LOG_DEBUG = (1 << 0),
	TOYBRICK_LOG_INFO = (1 << 1),
	TOYBRICK_LOG_WARNING = (1 << 2),
	TOYBRICK_LOG_ERROR = (1 << 3),
	TOYBRICK_LOG_PREFIX = (1 << 7),
};

static inline void colored_fputs(int level, const char *str)
{
	switch(level) {
	case TOYBRICK_LOG_DEBUG:
	case TOYBRICK_LOG_INFO:
		printf("%s", str);
		break;
	case TOYBRICK_LOG_WARNING:
		printf("\033[1;33m%s\033[0m", str);
		break;
	case TOYBRICK_LOG_ERROR:
		printf("\033[1;31m%s\033[0m", str);
		break;
	case TOYBRICK_LOG_PREFIX:
		printf("\033[1;34m%s\033[0m", str);
		break;
	default:
		break;
	}
}

static inline void toybrick_vlog(int level, const char* fmt, va_list vl)
{
	char str[512];

	vsnprintf(str, sizeof(str), fmt, vl);
	colored_fputs(TOYBRICK_LOG_PREFIX, "[Toybrick] ");
	colored_fputs(level, str);
}

static inline void toybrick_log(int level, const char *fmt, ...)
{
	va_list vl;
	int mask = TOYBRICK_DEFAULT_MASK;
	char *env = getenv("toybrick_log_mask");

	if(env)
		sscanf(env, "%x", &mask);

	if(!(mask & level))
		return;

	va_start(vl, fmt);
	toybrick_vlog(level, fmt, vl);
	va_end(vl);
}

#define toybrick_debug(fmt, ...) toybrick_log(TOYBRICK_LOG_DEBUG, fmt, ## __VA_ARGS__)
#define toybrick_info(fmt, ...) toybrick_log(TOYBRICK_LOG_INFO, fmt, ## __VA_ARGS__)
#define toybrick_warn(fmt, ...) toybrick_log(TOYBRICK_LOG_WARNING, fmt, ## __VA_ARGS__)
#define toybrick_err(fmt, ...) toybrick_log(TOYBRICK_LOG_ERROR, fmt, ## __VA_ARGS__)
#endif

#endif
